@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -13,21 +13,19 @@ import logging
 import os
 import traceback
 from functools import partial
+from urllib.parse import urljoin
 
 import requests
 from bravado.exception import HTTPError
-from reana_client.config import ERROR_MESSAGES, WORKFLOW_ENGINES
+from reana_client.config import ERROR_MESSAGES
 from reana_client.errors import FileDeletionError, FileUploadError
 from reana_client.utils import _validate_reana_yaml, is_uuid_v4
+from reana_commons.validation import validate_workflow_name
 from reana_commons.api_client import get_current_api_client
+from reana_commons.config import REANA_WORKFLOW_ENGINES
 from reana_commons.errors import REANASecretAlreadyExists, REANASecretDoesNotExist
 from werkzeug.local import LocalProxy
 
-try:
-    from urllib.parse import urljoin
-except ImportError:
-    from urlparse import urljoin
-
 current_rs_api_client = LocalProxy(
     partial(get_current_api_client, component="reana-server")
 )
@@ -63,8 +61,43 @@ def ping(access_token):
         return {"status": "ERROR: INVALID SERVER", "error": True}
 
 
+def get_user_quota(access_token):
+    """Retrieve user quota usage and limits."""
+    try:
+        response, http_response = current_rs_api_client.api.get_you(
+            access_token=access_token
+        ).result()
+        if http_response.status_code == 200:
+            return response["quota"]
+        raise Exception(
+            "Expected status code 200 but replied with "
+            "{status_code}".format(status_code=http_response.status_code)
+        )
+
+    except HTTPError as e:
+        logging.debug(
+            "User quotas could not be retrieved: "
+            "\nStatus: {}\nReason: {}\n"
+            "Message: {}".format(
+                e.response.status_code, e.response.reason, e.response.json()["message"]
+            )
+        )
+        raise Exception(e.response.json()["message"])
+    except Exception as e:
+        raise e
+
+
 def get_workflows(
-    access_token, type, verbose=False, block_size=None, page=None, size=None
+    access_token,
+    type,
+    verbose=False,
+    page=None,
+    size=None,
+    status=None,
+    search=None,
+    include_progress=None,
+    include_workspace_size=None,
+    workflow=None,
 ):
     """List all existing workflows."""
     try:
@@ -72,9 +105,13 @@ def get_workflows(
             access_token=access_token,
             verbose=verbose,
             type=type,
-            block_size=block_size,
             page=page,
             size=size,
+            status=status,
+            search=search,
+            include_progress=include_progress,
+            include_workspace_size=include_workspace_size,
+            workflow_id_or_name=workflow,
         ).result()
         if http_response.status_code == 200:
             return response.get("items")
@@ -189,6 +226,7 @@ def create_workflow_from_json(
             parameters={'files': ['file.txt'], 'parameters': {'key': 'value'}},
             workflow_engine='serial')
     """
+    validate_workflow_name(name)
     if is_uuid_v4(name):
         raise ValueError("Workflow name cannot be a valid UUIDv4")
     if not access_token:
@@ -196,10 +234,10 @@ def create_workflow_from_json(
     if os.environ.get("REANA_SERVER_URL") is None:
         raise Exception("Environment variable REANA_SERVER_URL is not set")
     workflow_engine = workflow_engine.lower()
-    if workflow_engine not in WORKFLOW_ENGINES:
+    if workflow_engine not in REANA_WORKFLOW_ENGINES:
         raise Exception(
             "Workflow engine - {} not found. You must use one of "
-            "these engines - {}".format(workflow_engine, WORKFLOW_ENGINES)
+            "these engines - {}".format(workflow_engine, REANA_WORKFLOW_ENGINES)
         )
     try:
         reana_yaml = dict(workflow={})
@@ -446,7 +484,9 @@ def delete_file(workflow, file_name, access_token):
         raise e
 
 
-def list_files(workflow, access_token, file_name=None, page=None, size=None):
+def list_files(
+    workflow, access_token, file_name=None, page=None, size=None, search=None
+):
     """Return the list of files for a given workflow workspace.
 
     :param workflow: name or id which identifies the workflow.
@@ -454,6 +494,7 @@ def list_files(workflow, access_token, file_name=None, page=None, size=None):
     :param file_name: file name(s) (glob) to list.
     :param page: page number of returned file list.
     :param size: page size of returned file list.
+    :param search: filter search results by parameters.
     :returns: a list of dictionaries composed by the `name`, `size` and
                 `last-modified`.
     """
@@ -464,6 +505,7 @@ def list_files(workflow, access_token, file_name=None, page=None, size=None):
             file_name=file_name,
             page=page,
             size=size,
+            search=search,
         ).result()
 
         if http_response.status_code == 200:
@@ -621,13 +663,12 @@ def get_workflow_specification(workflow, access_token):
         raise e
 
 
-def delete_workflow(workflow, all_runs, hard_delete, workspace, access_token):
+def delete_workflow(workflow, all_runs, workspace, access_token):
     """Delete a workflow."""
     try:
         parameters = {
-            "all_runs": True if all_runs == 1 else False,
-            "hard_delete": True if hard_delete == 1 else False,
-            "workspace": True if hard_delete == 1 or workspace == 1 else False,
+            "all_runs": all_runs,
+            "workspace": workspace,
         }
         (response, http_response) = current_rs_api_client.api.set_workflow_status(
             workflow_id_or_name=workflow,
@@ -695,8 +736,8 @@ def diff_workflows(workflow_id_a, workflow_id_b, brief, access_token, context_li
     :param context_lines: Optional parameter to set the number of
                           context lines shown in the diff output.
     :param access_token: API token of user requesting diff.
-    :returns: A list of dictionaries composed by `asset`, `type`, `lines`,
-        `a` and `b`. Asset refers to the workflow asset where a
+    :returns: A list of dictionaries composed by ``asset``, ``type``, ``lines``,
+        ``a`` and ``b``. Asset refers to the workflow asset where a
         difference was found, type refers to the asset type, lines refer
         to the lines of the file where the differences are and a, b fields
         are the actual lines that differ.
@@ -977,3 +1018,34 @@ def list_secrets(access_token):
         raise Exception(e.response.json()["message"])
     except Exception as e:
         raise e
+
+
+def info(access_token):
+    """List general information about the cluster.
+
+    :param access_token: access token of the current user.
+
+    """
+    try:
+        (response, http_response) = current_rs_api_client.api.info(
+            access_token=access_token
+        ).result()
+        if http_response.status_code == 200:
+            return response
+        else:
+            raise Exception(
+                "Expected status code 200 but replied with "
+                "{status_code}".format(status_code=http_response.status_code)
+            )
+
+    except HTTPError as e:
+        logging.debug(
+            "Workspaces could not be listed: "
+            "\nStatus: {}\nReason: {}\n"
+            "Message: {}".format(
+                e.response.status_code, e.response.reason, e.response.json()["message"]
+            )
+        )
+        raise Exception(e.response.json()["message"])
+    except Exception as e:
+        raise e
