@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2017, 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -13,7 +13,7 @@ import sys
 import click
 from urllib3 import disable_warnings
 
-from reana_client.cli import workflow, files, ping, secrets
+from reana_client.cli import workflow, files, ping, secrets, quotas
 from reana_client.utils import get_api_url
 
 DEBUG_LOG_FORMAT = (
@@ -37,6 +37,7 @@ class ReanaCLI(click.Group):
     """REANA command line interface."""
 
     cmd_groups = [
+        quotas.quota_group,
         ping.configuration_group,
         workflow.workflow_management_group,
         workflow.workflow_execution_group,
