@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -13,23 +13,37 @@ import os
 import re
 import sys
 import traceback
-import urllib
 from time import sleep
 
 import click
 import yaml
 from bravado.exception import HTTPServerError
-from cwltool.context import LoadingContext
 from cwltool.load_tool import fetch_document
 from cwltool.main import printdeps
-from cwltool.workflow import findfiles
 
 from reana_client.cli.utils import add_access_token_options
-from reana_client.config import default_user
 from reana_client.utils import load_workflow_spec
 from reana_client.version import __version__
 
-PY3 = sys.version_info > (3,)
+
+def findfiles(wo, fn=None):
+    """Return a list CWL workflow files."""
+    if fn is None:
+        fn = []
+    if isinstance(wo, dict):
+        if wo.get("class") == "File":
+            fn.append(wo)
+            findfiles(wo.get("secondaryFiles"), fn)
+        elif wo.get("class") == "Directory":
+            fn.append(wo)
+            findfiles(wo.get("secondaryFiles"), fn)
+        else:
+            for w in wo.values():
+                findfiles(w, fn)
+    elif isinstance(wo, list):
+        for w in wo:
+            findfiles(w, fn)
+    return fn
 
 
 def get_file_dependencies_obj(cwl_obj, basedir):
@@ -39,17 +53,19 @@ def get_file_dependencies_obj(cwl_obj, basedir):
     :param basedir: Workflow base dir.
     :returns: A dictionary composed of valid CWL file dependencies.
     """
-    # Load de document
-    loading_context = LoadingContext()
-    document_loader, workflow_obj, uri = fetch_document(
-        cwl_obj,
-        resolver=loading_context.resolver,
-        fetcher_constructor=loading_context.fetcher_constructor,
-    )
-    in_memory_buffer = io.StringIO() if PY3 else io.BytesIO()
+    # Load the document
+    # remove filename additions (e.g. 'v1.0/conflict-wf.cwl#collision')
+    document = cwl_obj.split("#")[0]
+    document_loader, workflow_obj, uri = fetch_document(document)
+    in_memory_buffer = io.StringIO()
     # Get dependencies
     printdeps(
-        workflow_obj, document_loader, in_memory_buffer, "primary", uri, basedir=basedir
+        workflow_obj,
+        document_loader.loader,
+        in_memory_buffer,
+        "primary",
+        uri,
+        basedir=basedir,
     )
     file_dependencies_obj = yaml.load(
         in_memory_buffer.getvalue(), Loader=yaml.FullLoader
@@ -58,6 +74,27 @@ def get_file_dependencies_obj(cwl_obj, basedir):
     return file_dependencies_obj
 
 
+def upload_files(files, basedir, workflow_id, access_token):
+    """Upload file or directory to REANA server."""
+    from reana_client.api.client import upload_file
+
+    for cwl_file_object in files:
+        file_path = cwl_file_object.get("location")
+        abs_file_path = os.path.join(basedir, file_path)
+
+        if os.path.isdir(abs_file_path):
+            for root, dirs, files in os.walk(abs_file_path, topdown=False):
+                for next_path in files + dirs:
+                    location = os.path.join(root, next_path).replace(basedir + "/", "")
+                    upload_files(
+                        [{"location": location}], basedir, workflow_id, access_token,
+                    )
+        else:
+            with open(abs_file_path, "r") as f:
+                upload_file(workflow_id, f, file_path, access_token)
+                logging.error("File {} uploaded.".format(file_path))
+
+
 @click.command()
 @click.version_option(version=__version__)
 @click.option("--quiet", is_flag=True, help="No diagnostic output")
@@ -68,11 +105,12 @@ def get_file_dependencies_obj(cwl_obj, basedir):
 )
 @click.option("--basedir", type=click.Path(), help="Base directory.")
 @add_access_token_options
-@click.argument("processfile", required=False)
-@click.argument("jobfile")
+@click.argument("processfile")
+@click.argument("jobfile", required=False)
 @click.pass_context
 def cwl_runner(ctx, quiet, outdir, basedir, processfile, jobfile, access_token):
     """Run CWL files in a standard format <workflow.cwl> <job.json>."""
+    import json
     from reana_client.utils import get_api_url
     from reana_client.api.client import (
         create_workflow,
@@ -88,52 +126,37 @@ def cwl_runner(ctx, quiet, outdir, basedir, processfile, jobfile, access_token):
     )
     try:
         basedir = basedir or os.path.abspath(os.path.dirname(processfile))
-        if processfile:
-            with open(jobfile) as f:
-                reana_spec = {
-                    "workflow": {"type": "cwl"},
-                    "inputs": {
-                        "parameters": {"input": yaml.load(f, Loader=yaml.FullLoader)}
-                    },
-                }
-
-            reana_spec["workflow"]["spec"] = load_workflow_spec(
-                reana_spec["workflow"]["type"], processfile,
-            )
-        else:
+        reana_spec = {"workflow": {"type": "cwl"}}
+        job = {}
+        if jobfile:
             with open(jobfile) as f:
                 job = yaml.load(f, Loader=yaml.FullLoader)
-            reana_spec = {"workflow": {"type": "cwl"}, "parameters": {"input": ""}}
 
-            reana_spec["workflow"]["spec"] = load_workflow_spec(
-                reana_spec["workflow"]["type"], job["cwl:tool"]
+        if processfile:
+            reana_spec["inputs"] = {"parameters": job}
+            reana_spec["workflow"]["specification"] = load_workflow_spec(
+                reana_spec["workflow"]["type"], processfile
             )
-            del job["cwl:tool"]
-            reana_spec["inputs"]["parameters"] = {"input": job}
-        reana_spec["workflow"]["spec"] = replace_location_in_cwl_spec(
-            reana_spec["workflow"]["spec"]
+        reana_spec["workflow"]["specification"] = replace_location_in_cwl_spec(
+            reana_spec["workflow"]["specification"]
         )
-
         logging.info("Connecting to {0}".format(get_api_url()))
-        response = create_workflow(reana_spec, "cwl-test", access_token)
+        reana_specification = json.loads(json.dumps(reana_spec, sort_keys=True))
+        response = create_workflow(reana_specification, "cwl-test", access_token)
         logging.error(response)
         workflow_name = response["workflow_name"]
         workflow_id = response["workflow_id"]
         logging.info(
             "Workflow {0}/{1} has been created.".format(workflow_name, workflow_id)
         )
-
         file_dependencies_list = []
         for cwlobj in [processfile, jobfile]:
-            file_dependencies_list.append(get_file_dependencies_obj(cwlobj, basedir))
+            if not cwlobj:
+                continue
+            file_dependencies_obj = get_file_dependencies_obj(cwlobj, basedir)
+            file_dependencies_list.append(file_dependencies_obj)
         files_to_upload = findfiles(file_dependencies_list)
-        for cwl_file_object in files_to_upload:
-            file_path = cwl_file_object.get("location")
-            abs_file_path = os.path.join(basedir, file_path)
-            with open(abs_file_path, "r") as f:
-                upload_file(workflow_id, f, file_path, access_token)
-                logging.error("File {} uploaded.".format(file_path))
-
+        upload_files(files_to_upload, basedir, workflow_id, access_token)
         response = start_workflow(
             workflow_id, access_token, reana_spec["inputs"]["parameters"]
         )
@@ -157,11 +180,14 @@ def cwl_runner(ctx, quiet, outdir, basedir, processfile, jobfile, access_token):
                 # click.echo(response['status'])
                 break
         try:
-            out = re.search(r"success{[\S\s]*", logs).group().replace("success", "")
             import ast
-            import json
 
-            json_output = json.dumps(ast.literal_eval(str(out)))
+            out = (
+                re.search(r"FinalOutput[\s\S]*?FinalOutput", logs)
+                .group()
+                .replace("FinalOutput", "")
+            )
+            json_output = out.encode("utf8").decode("unicode_escape")
         except AttributeError:
             logging.error("Workflow execution failed")
             sys.exit(1)
