@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2018, 2019, 2020, 2021 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -74,7 +74,7 @@ def files_group(ctx):
     help="Filter results to show only files that match certain filtering "
     "criteria such as file name, size or modification date."
     "Use `--filter <columm_name>=<column_value>` pairs. "
-    "Available filters are `name`, `size` and `last-modified`.",
+    "Available filters are ``name``, ``size`` and ``last-modified``.",
 )
 @click.argument("filename", metavar="SOURCE", nargs=1, required=False)
 @human_readable_or_raw_option
@@ -95,9 +95,9 @@ def get_files(
 ):  # noqa: D301
     """List workspace files.
 
-    The `ls` command lists workspace files of a workflow specified by the
+    The ``ls`` command lists workspace files of a workflow specified by the
     environment variable REANA_WORKON or provided as a command-line flag
-    `--workflow` or `-w`. The SOURCE argument is optional and specifies a
+    ``--workflow`` or ``-w``. The SOURCE argument is optional and specifies a
     pattern matching files and directories.
 
     Examples: \n
@@ -180,6 +180,7 @@ def get_files(
                 " for workflow {0}:\n{1}".format(workflow, str(e)),
                 msg_type="error",
             )
+            sys.exit(1)
 
 
 @files_group.command("download")
@@ -199,7 +200,7 @@ def download_files(
 ):  # noqa: D301
     """Download workspace files.
 
-    The `download` command allows to download workspace files and directories.
+    The ``download`` command allows to download workspace files and directories.
     By default, the files specified in the workflow specification as outputs
     are downloaded. You can also specify the individual files you would like
     to download, see examples below.
@@ -215,13 +216,25 @@ def download_files(
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
 
     if not filenames:
-        reana_spec = get_workflow_specification(workflow, access_token)["specification"]
+        try:
+            reana_spec = get_workflow_specification(workflow, access_token)[
+                "specification"
+            ]
+        except Exception as e:
+            logging.debug(str(e), exc_info=True)
+            display_message(
+                "Workflow {} could not be retrieved: {}".format(workflow, e),
+                msg_type="error",
+            )
+            sys.exit(1)
+
         if "outputs" in reana_spec:
             filenames = []
             filenames += reana_spec["outputs"].get("files", [])
             filenames += reana_spec["outputs"].get("directories", [])
 
     if workflow:
+        download_failed = False
         for file_name in filenames:
             try:
                 binary_file, file_name = download_file(
@@ -251,6 +264,7 @@ def download_files(
                     "File {0} could not be written.".format(file_name),
                     msg_type="error",
                 )
+                download_failed = True
             except Exception as e:
                 logging.debug(traceback.format_exc())
                 logging.debug(str(e))
@@ -258,6 +272,9 @@ def download_files(
                     "File {0} could not be downloaded: {1}".format(file_name, e),
                     msg_type="error",
                 )
+                download_failed = True
+        if download_failed:
+            sys.exit(1)
 
 
 @files_group.command("upload")
@@ -274,7 +291,7 @@ def download_files(
 def upload_files(ctx, workflow, filenames, access_token):  # noqa: D301
     """Upload files and directories to workspace.
 
-    The `upload` command allows to upload workflow input files and
+    The ``upload`` command allows to upload workflow input files and
     directories. The SOURCES argument can be repeated and specifies which files
     and directories are to be uploaded, see examples below. The default
     behaviour is to upload all input files and directories specified in the
@@ -290,7 +307,18 @@ def upload_files(ctx, workflow, filenames, access_token):  # noqa: D301
     for p in ctx.params:
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
     if not filenames:
-        reana_spec = get_workflow_specification(workflow, access_token)["specification"]
+        try:
+            reana_spec = get_workflow_specification(workflow, access_token)[
+                "specification"
+            ]
+        except Exception as e:
+            logging.debug(str(e), exc_info=True)
+            display_message(
+                "Workflow {} could not be retrieved: {}".format(workflow, e),
+                msg_type="error",
+            )
+            sys.exit(1)
+
         if "inputs" in reana_spec:
             filenames = []
             filenames += [
@@ -304,6 +332,7 @@ def upload_files(ctx, workflow, filenames, access_token):  # noqa: D301
 
     if workflow:
         if filenames:
+            upload_failed = False
             for filename in filenames:
                 try:
                     response = upload_to_server(workflow, filename, access_token)
@@ -327,18 +356,7 @@ def upload_files(ctx, workflow, filenames, access_token):  # noqa: D301
                         "{0} does not exist.".format(filename),
                         msg_type="error",
                     )
-                    if "invoked_by_subcommand" in ctx.parent.__dict__:
-                        sys.exit(1)
-                except FileUploadError as e:
-                    logging.debug(traceback.format_exc())
-                    logging.debug(str(e))
-                    display_message(
-                        "Something went wrong while uploading {0}.\n"
-                        "{1}".format(filename, str(e)),
-                        msg_type="error",
-                    )
-                    if "invoked_by_subcommand" in ctx.parent.__dict__:
-                        sys.exit(1)
+                    upload_failed = True
                 except Exception as e:
                     logging.debug(traceback.format_exc())
                     logging.debug(str(e))
@@ -347,8 +365,9 @@ def upload_files(ctx, workflow, filenames, access_token):  # noqa: D301
                         "{}".format(filename, str(e)),
                         msg_type="error",
                     )
-                    if "invoked_by_subcommand" in ctx.parent.__dict__:
-                        sys.exit(1)
+                    upload_failed = True
+            if upload_failed:
+                sys.exit(1)
 
 
 @files_group.command("rm")
@@ -360,7 +379,7 @@ def upload_files(ctx, workflow, filenames, access_token):  # noqa: D301
 def delete_files(ctx, workflow, filenames, access_token):  # noqa: D301
     """Delete files from workspace.
 
-    The `rm` command allow to delete files and directories from workspace.
+    The ``rm`` command allow to delete files and directories from workspace.
     Note that you can use glob to remove similar files.
 
     Examples:\n
@@ -374,6 +393,7 @@ def delete_files(ctx, workflow, filenames, access_token):  # noqa: D301
         logging.debug("{param}: {value}".format(param=p, value=ctx.params[p]))
 
     if workflow:
+        delete_failed = False
         for filename in filenames:
             try:
                 response = delete_file(workflow, filename, access_token)
@@ -389,14 +409,14 @@ def delete_files(ctx, workflow, filenames, access_token):  # noqa: D301
                         "{}".format(file_, response["failed"][file_]["error"]),
                         msg_type="error",
                     )
+                    delete_failed = True
                 if freed_space:
                     display_message(
                         f"{freed_space} bytes freed up.", msg_type="success"
                     )
             except FileDeletionError as e:
                 display_message(str(e), msg_type="error")
-                if "invoked_by_subcommand" in ctx.parent.__dict__:
-                    sys.exit(1)
+                delete_failed = True
             except Exception as e:
                 logging.debug(traceback.format_exc())
                 logging.debug(str(e))
@@ -404,8 +424,9 @@ def delete_files(ctx, workflow, filenames, access_token):  # noqa: D301
                     "Something went wrong while deleting {}".format(filename),
                     msg_type="error",
                 )
-                if "invoked_by_subcommand" in ctx.parent.__dict__:
-                    sys.exit(1)
+                delete_failed = True
+        if delete_failed:
+            sys.exit(1)
 
 
 @files_group.command("mv")
@@ -418,7 +439,7 @@ def delete_files(ctx, workflow, filenames, access_token):  # noqa: D301
 def move_files(ctx, source, target, workflow, access_token):  # noqa: D301
     """Move files within workspace.
 
-    The `mv` command allow to move the files within workspace.
+    The ``mv`` command allow to move the files within workspace.
 
     Examples:\n
     \t $ reana-client mv data/input.txt input/input.txt
@@ -455,6 +476,7 @@ def move_files(ctx, source, target, workflow, access_token):  # noqa: D301
             logging.debug(traceback.format_exc())
             logging.debug(str(e))
             display_message("Something went wrong. {}".format(e), msg_type="error")
+            sys.exit(1)
 
 
 @files_group.command("du")
@@ -469,7 +491,7 @@ def move_files(ctx, source, target, workflow, access_token):  # noqa: D301
     help="Filter results to show only files that match certain filtering "
     "criteria such as file name or size."
     "Use `--filter <columm_name>=<column_value>` pairs. "
-    "Available filters are `name` and `size`.",
+    "Available filters are ``name`` and ``size``.",
 )
 @human_readable_or_raw_option
 @click.pass_context
@@ -478,7 +500,7 @@ def workflow_disk_usage(
 ):  # noqa: D301
     """Get workspace disk usage.
 
-    The `du` command allows to chech the disk usage of given workspace.
+    The ``du`` command allows to chech the disk usage of given workspace.
 
     Examples: \n
     \t $ reana-client du -w myanalysis.42 -s \n
@@ -518,3 +540,4 @@ def workflow_disk_usage(
             display_message(
                 "Disk usage could not be retrieved: \n{}".format(e), msg_type="error",
             )
+            sys.exit(1)
