@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2018 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -12,22 +12,30 @@ import json
 import os
 import shlex
 import sys
+from typing import Callable, NoReturn, Optional, Union
 
 import click
+from reana_commons.validation import validate_workflow_name
 
+from reana_client.config import (
+    ERROR_MESSAGES,
+    RUN_STATUSES,
+    JOB_STATUS_TO_MSG_COLOR,
+)
+from reana_client.printer import display_message
 from reana_client.utils import workflow_uuid_or_name
-from reana_client.config import ERROR_MESSAGES
-from reana_commons.errors import MissingAPIClientConfiguration
 
 
-def add_access_token_options(func):
+def _access_token_option_decorator(func: Callable, required: bool) -> Callable:
     """Add access token related options to click commands."""
 
     @click.option(
         "-t",
         "--access-token",
-        default=os.getenv("REANA_ACCESS_TOKEN", None),
-        callback=access_token_check,
+        default=os.getenv("REANA_ACCESS_TOKEN"),
+        callback=lambda ctx, _, access_token: access_token_check(
+            ctx, _, access_token, required
+        ),
         help="Access token of the current user.",
     )
     @functools.wraps(func)
@@ -37,12 +45,42 @@ def add_access_token_options(func):
     return wrapper
 
 
-def access_token_check(ctx, _, access_token):
+add_access_token_options = functools.partial(
+    _access_token_option_decorator, required=True
+)
+add_access_token_options_not_required = functools.partial(
+    _access_token_option_decorator, required=False
+)
+
+
+def human_readable_or_raw_option(func):
+    """Add human readable option to click commands."""
+
+    @click.option(
+        "-h",
+        "--human-readable",
+        "human_readable_or_raw",
+        is_flag=True,
+        default=False,
+        callback=lambda ctx, param, value: "human_readable" if value else "raw",
+        help="Show disk size in human readable format.",
+    )
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        return func(*args, **kwargs)
+
+    return wrapper
+
+
+def access_token_check(
+    ctx: click.core.Context,
+    _: click.core.Option,
+    access_token: Optional[str],
+    required: bool,
+) -> Union[str, NoReturn]:
     """Check if access token is present."""
-    if not access_token:
-        click.echo(
-            click.style(ERROR_MESSAGES["missing_access_token"], fg="red"), err=True
-        )
+    if not access_token and required:
+        display_message(ERROR_MESSAGES["missing_access_token"], msg_type="error")
         ctx.exit(1)
     else:
         return access_token
@@ -57,10 +95,8 @@ def check_connection(func):
 
         api_url = get_api_url()
         if not api_url:
-            click.secho(
-                "REANA client is not connected to any REANA cluster.",
-                fg="red",
-                err=True,
+            display_message(
+                "REANA client is not connected to any REANA cluster.", msg_type="error",
             )
             sys.exit(1)
         return func(*args, **kwargs)
@@ -105,11 +141,11 @@ def add_pagination_options(func):
     return wrapper
 
 
-def parse_parameters(_filter):
+def parse_format_parameters(_format):
     """Return parsed filter parameters."""
     try:
         parsed_filters = []
-        filters = " ".join(_filter).replace(",", " ")
+        filters = " ".join(_format).replace(",", " ")
         for item in shlex.split(filters):
             if "=" in item:
                 filter_item = {
@@ -121,13 +157,56 @@ def parse_parameters(_filter):
             parsed_filters.append(filter_item)
         return parsed_filters
     except ValueError as e:
-        click.echo(
-            click.style("Wrong filter format \n{0}".format(e.message), fg="red"),
-            err=True,
+        display_message(
+            "Wrong filter format \n{0}".format(e.message), msg_type="error",
         )
 
 
-def filter_data(parsed_filters, headers, tablib_data):
+def parse_filter_parameters(filters, filter_names):
+    """Return parsed filter parameters."""
+    try:
+        status_filters = []
+        search_filters = {}
+        filters = list(filters)
+        for item in filters:
+            if "=" in item:
+                filter_name = item.split("=")[0].lower()
+                filter_value = item.split("=")[1]
+                if filter_name == "status" and filter_name in filter_names:
+                    if filter_value in RUN_STATUSES:
+                        status_filters.append(filter_value)
+                    else:
+                        display_message(
+                            "Input status value {} is not valid. ".format(filter_value),
+                            msg_type="error",
+                        ),
+                        sys.exit(1)
+                elif filter_name in filter_names:
+                    if search_filters.get(filter_name):
+                        search_filters[filter_name].append(filter_value)
+                    else:
+                        search_filters[filter_name] = [filter_value]
+                else:
+                    display_message(
+                        "Filter {} is not valid.".format(filter_name), msg_type="error",
+                    ),
+                    sys.exit(1)
+            else:
+                raise click.BadParameter(
+                    "Wrong input format. Please use --filter filter_name=filter_value"
+                )
+        if not search_filters:
+            search_filters = None
+        else:
+            search_filters = json.dumps(search_filters)
+        return status_filters, search_filters
+    except ValueError as e:
+        display_message(
+            "Wrong filter format \n{0}".format(e.message), msg_type="error",
+        )
+
+
+def format_data(parsed_filters, headers, tablib_data):
     """Return filtered data."""
     parsed_filters = [i for i in parsed_filters if i["column_name"] in headers]
     column_headers = [i["column_name"] for i in parsed_filters] or None
@@ -141,6 +220,7 @@ def filter_data(parsed_filters, headers, tablib_data):
                 and filter_["column_value"] != item[filter_["column_name"]]
             ):
                 tablib_data.remove(item)
+                break
     return tablib_data, column_headers or []
 
 
@@ -151,22 +231,22 @@ def format_session_uri(reana_server_url, path, access_token):
     )
 
 
-def validate_workflow_name(ctx, _, workflow_name):
-    """Validate workflow name."""
-    not_allowed_characters = ["."]
-    if workflow_name:
-        for item in not_allowed_characters:
-            if item in workflow_name:
-                click.echo(
-                    click.style(
-                        "Workflow name {} contains illegal "
-                        'character "{}"'.format(workflow_name, item),
-                        fg="red",
-                    ),
-                    err=True,
-                )
-                sys.exit(1)
-    return workflow_name
+def get_formatted_progress(progress):
+    """Return workflow progress in format of finished/total jobs."""
+    total_jobs = progress.get("total", {}).get("total") or "-"
+    finished_jobs = progress.get("finished", {}).get("total") or "-"
+    return "{0}/{1}".format(finished_jobs, total_jobs)
+
+
+def validate_workflow_name_parameter(
+    ctx: click.core.Context, _: click.core.Option, workflow_name: str
+) -> Union[str, NoReturn]:
+    """Validate workflow name parameter."""
+    try:
+        return validate_workflow_name(workflow_name)
+    except ValueError as e:
+        display_message(str(e), msg_type="error")
+        sys.exit(1)
 
 
 def key_value_to_dict(ctx, param, value):
@@ -178,11 +258,10 @@ def key_value_to_dict(ctx, param, value):
     try:
         return dict(op.split("=") for op in value)
     except ValueError:
-        click.secho(
-            '==> ERROR: Input parameter "{0}" is not valid. '
+        display_message(
+            'Input parameter "{0}" is not valid. '
             'It must follow format "param=value".'.format(" ".join(value)),
-            err=True,
-            fg="red",
+            msg_type="error",
         ),
         sys.exit(1)
 
@@ -190,12 +269,9 @@ def key_value_to_dict(ctx, param, value):
 def requires_environments(ctx, param, value):
     """Require passing ``--environments`` flag."""
     if value and not ctx.params.get("environments"):
-        click.secho(
-            "==> ERROR: `{}` flag requires `--environments` flag.".format(
-                param.opts[0]
-            ),
-            err=True,
-            fg="red",
+        display_message(
+            "`{}` flag requires `--environments` flag.".format(param.opts[0]),
+            msg_type="error",
         )
         sys.exit(1)
     return value
@@ -215,15 +291,11 @@ class NotRequiredIf(click.Option):
         argument_present = self.name in opts
         other_argument_present = self.not_required_if in opts
         if not argument_present and not other_argument_present:
-            click.echo(
-                click.style(
-                    "At least one of the options: `{}` or `{}` is required\n".format(
-                        self.name, self.not_required_if
-                    ),
-                    fg="red",
-                )
+            display_message(
+                "At least one of the options: `{}` or `{}` "
+                "is required\n".format(self.name, self.not_required_if)
                 + ctx.get_help(),
-                err=True,
+                msg_type="error",
             )
             sys.exit(1)
 
@@ -237,12 +309,6 @@ def output_user_friendly_logs(workflow_logs, steps):
         are returned from the REST API.
     :param steps: List of steps to show logs for.
     """
-    # Colors matching the status colors of REANA-UI
-    status_to_color_mapping = {
-        "running": "bright_blue",
-        "finished": "green",
-        "failed": "red",
-    }
     key_to_description_mapping = {
         "workflow_uuid": "Workflow ID",
         "compute_backend": "Compute backend",
@@ -250,19 +316,19 @@ def output_user_friendly_logs(workflow_logs, steps):
         "docker_img": "Docker image",
         "cmd": "Command",
         "status": "Status",
+        "started_at": "Started",
+        "finished_at": "Finished",
     }
     leading_mark = "==>"
 
     # REANA Workflow Engine logs
     if workflow_logs.get("workflow_logs"):
-        click.secho(
-            "{} Workflow engine logs".format(leading_mark), bold=True, fg="yellow"
-        )
+        click.secho(f"{leading_mark} Workflow engine logs", bold=True, fg="yellow")
         click.echo(workflow_logs["workflow_logs"])
 
     if workflow_logs.get("engine_specific"):
         click.echo("\n")
-        click.secho("{} Engine internal logs".format(leading_mark), fg="yellow")
+        click.secho(f"{leading_mark} Engine internal logs", fg="yellow")
         click.secho(workflow_logs["engine_specific"])
 
     returned_step_names = set(
@@ -272,13 +338,11 @@ def output_user_friendly_logs(workflow_logs, steps):
     if steps:
         missing_steps = set(steps).difference(returned_step_names)
         if missing_steps:
-            click.echo(
-                click.style(
-                    "The logs of step(s) {} were not found, "
-                    "check for spelling mistakes in the step "
-                    "names.".format(",".join(missing_steps)),
-                    fg="red",
-                )
+            display_message(
+                "The logs of step(s) {} were not found, "
+                "check for spelling mistakes in the step "
+                "names.".format(",".join(missing_steps)),
+                msg_type="error",
             )
     # Job logs
     if workflow_logs["job_logs"]:
@@ -287,10 +351,11 @@ def output_user_friendly_logs(workflow_logs, steps):
     for job_id, logs_info in workflow_logs["job_logs"].items():
         if logs_info:
             job_name_or_id = logs_info["job_name"] or job_id
+
             click.secho(
-                "{0} Step: {1}".format(leading_mark, job_name_or_id),
+                f"{leading_mark} Step: {job_name_or_id}",
                 bold=True,
-                fg=status_to_color_mapping.get(logs_info["status"]),
+                fg=JOB_STATUS_TO_MSG_COLOR.get(logs_info["status"]),
             )
             logs_output = logs_info["logs"]
             # extract already used fields
@@ -298,24 +363,20 @@ def output_user_friendly_logs(workflow_logs, steps):
             del logs_info["job_name"]
 
             for key, value in logs_info.items():
-                title = click.style(
-                    "{mark} {description}:".format(
-                        mark=leading_mark, description=key_to_description_mapping[key]
-                    ),
-                    fg=status_to_color_mapping.get(logs_info["status"]),
-                )
-                click.secho("{title} {value}".format(title=title, value=value))
+                if value:
+                    title = click.style(
+                        f"{leading_mark} {key_to_description_mapping[key]}:",
+                        fg=JOB_STATUS_TO_MSG_COLOR.get(logs_info["status"]),
+                    )
+                    click.echo(f"{title} {value}")
             # show actual log content
             if logs_output:
                 click.secho(
-                    "{mark} Logs:".format(mark=leading_mark),
-                    fg=status_to_color_mapping.get(logs_info["status"]),
+                    f"{leading_mark} Logs:",
+                    fg=JOB_STATUS_TO_MSG_COLOR.get(logs_info["status"]),
                 )
                 click.secho(logs_output)
             else:
-                click.secho(
-                    "{0} Step {1} emitted no logs.".format(
-                        leading_mark, job_name_or_id
-                    ),
-                    bold=True,
+                display_message(
+                    f"Step {job_name_or_id} emitted no logs.", msg_type="info",
                 )
