@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018, 2019, 2020, 2021 CERN.
+# Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -36,7 +36,7 @@ from reana_client.cli.utils import (
     parse_filter_parameters,
     parse_format_parameters,
     requires_environments,
-    validate_workflow_name,
+    validate_workflow_name_parameter,
 )
 from reana_client.config import ERROR_MESSAGES, RUN_STATUSES, TIMECHECK
 from reana_client.printer import display_message
@@ -114,7 +114,7 @@ def workflow_execution_group(ctx):
     multiple=True,
     help="Filter workflow that contains certain filtering criteria. "
     "Use `--filter <columm_name>=<column_value>` pairs. "
-    "Available filters are `name` and `status`.",
+    "Available filters are ``name`` and ``status``.",
 )
 @click.option(
     "--include-progress",
@@ -153,9 +153,9 @@ def workflow_workflows(  # noqa: C901
 ):  # noqa: D301
     """List all workflows and sessions.
 
-    The `list` command lists workflows and sessions. By default, the list of
+    The ``list`` command lists workflows and sessions. By default, the list of
     workflows is returned. If you would like to see the list of your open
-    interactive sessions, you need to pass the `--sessions` command-line
+    interactive sessions, you need to pass the ``--sessions`` command-line
     option.
 
     Example: \n
@@ -277,6 +277,7 @@ def workflow_workflows(  # noqa: C901
             "Workflow list could not be retrieved: \n{}".format(str(e)),
             msg_type="error",
         )
+        sys.exit(1)
 
 
 @workflow_management_group.command("create")
@@ -294,7 +295,7 @@ def workflow_workflows(  # noqa: C901
     "-w",
     "--workflow",
     default="",
-    callback=validate_workflow_name,
+    callback=validate_workflow_name_parameter,
     help='Optional name of the workflow. [default is "workflow"]',
 )
 @click.option(
@@ -309,7 +310,7 @@ def workflow_workflows(  # noqa: C901
 def workflow_create(ctx, file, name, skip_validation, access_token):  # noqa: D301
     """Create a new workflow.
 
-    The `create` command allows to create a new workflow from reana.yaml
+    The ``create`` command allows to create a new workflow from reana.yaml
     specifications file. The file is expected to be located in the current
     working directory, or supplied via command-line -f option, see examples
     below.
@@ -331,11 +332,14 @@ def workflow_create(ctx, file, name, skip_validation, access_token):  # noqa: D3
     # could be made between the name and actual UUID of workflow.
     if is_uuid_v4(name):
         display_message("Workflow name cannot be a valid UUIDv4", msg_type="error")
+        sys.exit(1)
+
     try:
         reana_specification = load_reana_spec(
             click.format_filename(file),
             access_token=access_token,
             skip_validation=skip_validation,
+            server_capabilities=True,
         )
         logging.info("Connecting to {0}".format(get_api_url()))
         response = create_workflow(reana_specification, name, access_token)
@@ -349,8 +353,7 @@ def workflow_create(ctx, file, name, skip_validation, access_token):  # noqa: D3
         display_message(
             "Cannot create workflow {}: \n{}".format(name, str(e)), msg_type="error"
         )
-        if "invoked_by_subcommand" in ctx.parent.__dict__:
-            sys.exit(1)
+        sys.exit(1)
 
 
 @workflow_execution_group.command("start")
@@ -390,12 +393,12 @@ def workflow_start(
 ):  # noqa: D301
     """Start previously created workflow.
 
-    The `start` command allows to start previously created workflow. The
+    The ``start`` command allows to start previously created workflow. The
     workflow execution can be further influenced by passing input prameters
-    using `-p` or `--parameters` flag and by setting additional operational
-    options using `-o` or `--options`.  The input parameters and operational
+    using ``-p`` or ``--parameters`` flag and by setting additional operational
+    options using ``-o`` or ``--options``.  The input parameters and operational
     options can be repetitive. For example, to disable caching for the Serial
-    workflow engine, you can set `-o CACHE=off`.
+    workflow engine, you can set ``-o CACHE=off``.
 
     Examples: \n
     \t $ reana-client start -w myanalysis.42 -p sleeptime=10 -p myparam=4 \n
@@ -435,6 +438,7 @@ def workflow_start(
                     "{0} \n{1}".format(parameters, str(e)),
                     msg_type="error",
                 )
+                sys.exit(1)
         try:
             logging.info("Connecting to {0}".format(get_api_url()))
             response = start_workflow(workflow, access_token, parsed_parameters)
@@ -474,8 +478,7 @@ def workflow_start(
                 "Cannot start workflow {}: \n{}".format(workflow, str(e)),
                 msg_type="error",
             )
-            if "invoked_by_subcommand" in ctx.parent.__dict__:
-                sys.exit(1)
+            sys.exit(1)
 
 
 @workflow_execution_group.command("restart")
@@ -515,16 +518,16 @@ def workflow_restart(
 ):  # noqa: D301
     """Restart previously run workflow.
 
-    The `restart` command allows to restart a previous workflow on the same
+    The ``restart`` command allows to restart a previous workflow on the same
     workspace.
 
     Note that workflow restarting can be used in a combination with operational
     options ``FROM`` and ``TARGET``. You can also pass a modified workflow
-    specification with ``-f`` or `--file`` flag.
+    specification with ``-f`` or ``--file`` flag.
 
-    You can furthermore use modified input prameters using `-p` or
-    `--parameters` flag and by setting additional operational options using
-    `-o` or `--options`.  The input parameters and operational options can be
+    You can furthermore use modified input prameters using ``-p`` or
+    ``--parameters`` flag and by setting additional operational options using
+    ``-o`` or ``--options``.  The input parameters and operational options can be
     repetitive.
 
     Examples: \n
@@ -586,6 +589,7 @@ def workflow_restart(
                     "{0} \n{1}".format(parameters, str(e)),
                     msg_type="error",
                 )
+                sys.exit(1)
         try:
             logging.info("Connecting to {0}".format(get_api_url()))
             response = start_workflow(workflow, access_token, parsed_parameters)
@@ -602,8 +606,7 @@ def workflow_restart(
                 "Cannot start workflow {}: \n{}".format(workflow, str(e)),
                 msg_type="error",
             )
-            if "invoked_by_subcommand" in ctx.parent.__dict__:
-                sys.exit(1)
+            sys.exit(1)
 
 
 @workflow_execution_group.command("status")
@@ -631,7 +634,7 @@ def workflow_status(  # noqa: C901
 ):  # noqa: D301
     """Get status of a workflow.
 
-    The `status` command allow to retrieve status of a workflow. The status can
+    The ``status`` command allow to retrieve status of a workflow. The status can
     be created, queued, running, failed, etc. You can increase verbosity or
     filter retrieved information by passing appropriate command-line options.
 
@@ -751,6 +754,7 @@ def workflow_status(  # noqa: C901
                 "{}".format(workflow, str(e)),
                 msg_type="error",
             )
+            sys.exit(1)
 
 
 @workflow_execution_group.command("logs")
@@ -776,9 +780,9 @@ def workflow_logs(
     page=None,
     size=None,
 ):  # noqa: D301
-    """Get  workflow logs.
+    """Get workflow logs.
 
-    The `logs` command allows to retrieve logs of running workflow. Note that
+    The ``logs`` command allows to retrieve logs of running workflow. Note that
     only finished steps of the workflow are returned, the logs of the currently
     processed step is not returned until it is finished.
 
@@ -878,6 +882,7 @@ def workflow_logs(
                 "{}".format(workflow, str(e)),
                 msg_type="error",
             )
+            sys.exit(1)
 
 
 @workflow_execution_group.command("validate")
@@ -918,7 +923,7 @@ def workflow_validate(
 ):  # noqa: D301
     """Validate workflow specification file.
 
-    The `validate` command allows to check syntax and validate the reana.yaml
+    The ``validate`` command allows to check syntax and validate the reana.yaml
     workflow specification file.
 
     Examples: \n
@@ -977,7 +982,7 @@ def workflow_validate(
 def workflow_stop(ctx, workflow, force_stop, access_token):  # noqa: D301
     """Stop a running workflow.
 
-    The `stop` command allows to hard-stop the running workflow process. Note
+    The ``stop`` command allows to hard-stop the running workflow process. Note
     that soft-stopping of the workflow is currently not supported. This command
     should be therefore used with care, only if you are absolutely sure that
     there is no point in continuing the running the workflow.
@@ -1010,6 +1015,7 @@ def workflow_stop(ctx, workflow, force_stop, access_token):  # noqa: D301
                 "Cannot stop workflow {}: \n{}".format(workflow, str(e)),
                 msg_type="error",
             )
+            sys.exit(1)
 
 
 @workflow_execution_group.command("run")
@@ -1027,7 +1033,7 @@ def workflow_stop(ctx, workflow, force_stop, access_token):  # noqa: D301
     "-w",
     "--workflow",
     default="",
-    callback=validate_workflow_name,
+    callback=validate_workflow_name_parameter,
     help='Optional name of the workflow. [default is "workflow"]',
 )
 @click.option(
@@ -1070,7 +1076,7 @@ def workflow_run(
 ):  # noqa: D301
     """Shortcut to create, upload, start a new workflow.
 
-    The `run` command allows to create a new workflow, upload its input files
+    The ``run`` command allows to create a new workflow, upload its input files
     and start it in one command.
 
     Examples: \n
@@ -1126,12 +1132,12 @@ def workflow_run(
 def workflow_delete(ctx, workflow, all_runs, workspace, access_token):  # noqa: D301
     """Delete a workflow.
 
-    The `delete` command allows to remove workflow runs from the database and
+    The ``delete`` command allows to remove workflow runs from the database and
     the workspace. By default, the command removes the workflow and all its
     cached information and hides the workflow from the workflow list. Note that
     workflow workspace will still be accessible until you use
-    `--include-workspace` flag. Note also that you can remove all past runs of
-    a workflow by specifying `--include-all-runs` flag.
+    ``--include-workspace`` flag. Note also that you can remove all past runs of
+    a workflow by specifying ``--include-all-runs`` flag.
 
     Example: \n
     \t $ reana-client delete -w myanalysis.42 \n
@@ -1164,6 +1170,7 @@ def workflow_delete(ctx, workflow, all_runs, workspace, access_token):  # noqa:
                 "Cannot delete workflow {} \n{}".format(workflow, str(e)),
                 msg_type="error",
             )
+            sys.exit(1)
 
 
 @workflow_management_group.command("diff")
@@ -1197,7 +1204,7 @@ def workflow_diff(
 ):  # noqa: D301
     """Show diff between two workflows.
 
-    The `diff` command allows to compare two workflows, the workflow_a and
+    The ``diff`` command allows to compare two workflows, the workflow_a and
     workflow_b, which must be provided as arguments. The output will show the
     difference in workflow run parameters, the generated files, the logs, etc.
 
@@ -1264,6 +1271,7 @@ def workflow_diff(
             "Something went wrong when trying to get diff:\n{}".format(str(e)),
             msg_type="error",
         )
+        sys.exit(1)
 
 
 @click.group(help="Workspace interactive commands")
@@ -1294,7 +1302,7 @@ def workflow_open_interactive_session(
 ):  # noqa: D301
     """Open an interactive session inside the workspace.
 
-    The `open` command allows to open interactive session processes on top of
+    The ``open`` command allows to open interactive session processes on top of
     the workflow workspace, such as Jupyter notebooks. This is useful to
     quickly inspect and analyse the produced files while the workflow is stlil
     running.
@@ -1337,6 +1345,7 @@ def workflow_open_interactive_session(
                 "Interactive session could not be opened: \n{}".format(str(e)),
                 msg_type="error",
             )
+            sys.exit(1)
     else:
         display_message("Cannot find workflow {}".format(workflow), msg_type="error")
 
@@ -1348,7 +1357,7 @@ def workflow_open_interactive_session(
 def workflow_close_interactive_session(workflow, access_token):  # noqa: D301
     """Close an interactive session.
 
-    The `close` command allows to shut down any interactive sessions that you
+    The ``close`` command allows to shut down any interactive sessions that you
     may have running. You would typically use this command after you finished
     exploring data in the Jupyter notebook and after you have transferred any
     code created in your interactive session.
@@ -1374,5 +1383,6 @@ def workflow_close_interactive_session(workflow, access_token):  # noqa: D301
                 "Interactive session could not be closed: \n{}".format(str(e)),
                 msg_type="error",
             )
+            sys.exit(1)
     else:
         display_message("Cannot find workflow {} ".format(workflow), msg_type="error")
