@@ -1,13 +1,12 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2017, 2018 CERN.
+# Copyright (C) 2017, 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
 """REANA client configuration."""
 
-import os
 
 import pkg_resources
 
@@ -37,8 +36,17 @@ TIMECHECK = 5
 URL = "url"
 """Url output format."""
 
-WORKFLOW_ENGINES = ["serial", "cwl", "yadage"]
-"""Supported workflow engines."""
+RUN_STATUSES = [
+    "created",
+    "running",
+    "finished",
+    "failed",
+    "deleted",
+    "stopped",
+    "queued",
+    "pending",
+]
+"""Available run statuses."""
 
 ENVIRONMENT_IMAGE_SUSPECTED_TAGS_VALIDATOR = ["latest", "master", ""]
 """Warns user if above environment image tags are used."""
@@ -68,3 +76,9 @@ PRINTER_COLOUR_ERROR = "red"
 
 PRINTER_COLOUR_INFO = "cyan"
 """Default colour for info messages on terminal."""
+
+JOB_STATUS_TO_MSG_COLOR = {
+    "failed": "red",
+    "finished": "green",
+    "running": "bright_blue",
+}
