@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2021 CERN.
+# Copyright (C) 2021, 2022 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -13,7 +13,12 @@ import logging
 
 import requests
 
-from reana_commons.config import WORKFLOW_RUNTIME_USER_GID, WORKFLOW_RUNTIME_USER_UID
+from reana_commons.config import (
+    REANA_DEFAULT_SNAKEMAKE_ENV_IMAGE,
+    WORKFLOW_RUNTIME_USER_GID,
+    WORKFLOW_RUNTIME_USER_UID,
+)
+
 
 from reana_client.errors import EnvironmentValidationError
 from reana_client.config import (
@@ -28,7 +33,7 @@ from reana_client.printer import display_message
 def validate_environment(reana_yaml, pull=False):
     """Validate environments in REANA specification file according to workflow type.
 
-    :param reana_yaml: Dictionary which represents REANA specifications file.
+    :param reana_yaml: Dictionary which represents REANA specification file.
     :param pull: If true, attempt to pull remote environment image to perform GID/UID validation.
     """
 
@@ -41,8 +46,13 @@ def validate_environment(reana_yaml, pull=False):
             workflow_steps = workflow["specification"]["stages"]
             return YadageEnvironmentValidator(workflow_steps=workflow_steps, pull=pull)
         if workflow_type == "cwl":
-            workflow_file = workflow.get("file")
-            return CWLEnvironmentValidator(workflow_file=workflow_file, pull=pull)
+            workflow_steps = workflow.get("specification", {}).get("$graph", workflow)
+            return CWLEnvironmentValidator(workflow_steps=workflow_steps, pull=pull)
+        if workflow_type == "snakemake":
+            workflow_steps = workflow["specification"]["steps"]
+            return SnakemakeEnvironmentValidator(
+                workflow_steps=workflow_steps, pull=pull
+            )
 
     workflow = reana_yaml["workflow"]
     validator = build_validator(workflow)
@@ -53,15 +63,13 @@ def validate_environment(reana_yaml, pull=False):
 class EnvironmentValidatorBase:
     """REANA workflow environments validation base class."""
 
-    def __init__(self, workflow_steps=None, workflow_file=None, pull=False):
+    def __init__(self, workflow_steps=None, pull=False):
         """Validate environments in REANA workflow.
 
         :param workflow_steps: List of dictionaries which represents different steps involved in workflow.
-        :param workflow_file: Path to workflow specification.
         :param pull: If true, attempt to pull remote environment image to perform GID/UID validation.
         """
         self.workflow_steps = workflow_steps
-        self.workflow_file = workflow_file
         self.pull = pull
         self.validated_images = set()
         self.messages = []
@@ -456,18 +464,36 @@ class CWLEnvironmentValidator(EnvironmentValidatorBase):
     def validate_environment(self):
         """Validate environments in REANA CWL workflow."""
 
-        try:
-            import cwl_utils.parser_v1_0 as cwl_parser
-            from cwl_utils.docker_extract import traverse
-        except ImportError as e:
-            display_message(
-                "Cannot validate environment. Please install reana-client on Python 3+ to enable environment validation for CWL workflows.",
-                msg_type="error",
-                indented=True,
-            )
-            raise e
+        def _validate_workflow_environment(workflow_steps):
+            """Validate environments in REANA CWL workflow steps."""
+            requirements = workflow_steps.get("requirements", [])
+            images = list(filter(lambda req: "dockerPull" in req, requirements))
 
-        top = cwl_parser.load_document(self.workflow_file)
+            for image in images:
+                self._validate_environment_image(image["dockerPull"])
 
-        for image in traverse(top):
-            self._validate_environment_image(image)
+        workflow = self.workflow_steps
+        if isinstance(workflow, dict):
+            _validate_workflow_environment(workflow)
+        elif isinstance(workflow, list):
+            for wf in workflow:
+                _validate_workflow_environment(wf)
+
+
+class SnakemakeEnvironmentValidator(EnvironmentValidatorBase):
+    """REANA Snakemake workflow environments validation."""
+
+    def validate_environment(self):
+        """Validate environments in REANA Snakemake workflow."""
+        for step in self.workflow_steps:
+            image = step["environment"]
+            if not image:
+                self.messages.append(
+                    {
+                        "type": "warning",
+                        "message": f"Environment image not specified, using {REANA_DEFAULT_SNAKEMAKE_ENV_IMAGE}.",
+                    }
+                )
+                image = REANA_DEFAULT_SNAKEMAKE_ENV_IMAGE
+            kubernetes_uid = step.get("kubernetes_uid")
+            self._validate_environment_image(image, kubernetes_uid=kubernetes_uid)
